!----*|--.---------.---------.---------.---------.---------.---------.-|-------|
subroutine courant
  !----*|--.---------.---------.---------.---------.---------.---------.-|-------|
  ! This subroutine sets the timestep based on the specified CFL number
  ! The subroutine should be called with the velocity in physical space

  include 'header'

  real(rkind) vel
  real(rkind) dt_x, dt_y, dt_z
  real(rkind) Nmax
  integer i, j, k, n
  integer imin, jmin, kmin

  ! Set the initial dt to some arbitrary large number
  dt = 1.d0

  ! Set the timestep based on viscosity and diffusivity
  dt = min(dt, 0.5d0 * min(dx(1), dy(1))**(2.d0 * beta) / nu)
  do n = 1, N_th
    dt = min(dt, dt * nu / (nu / Pr(n)))
  end do
  ! Make sure that we capture the inertial period (for rotating flows)
  if (Ro_inv /= 0.d0) then
    dt = min(dt, 2.d0 * pi / abs((Ro_inv / delta)) / 20.d0)
  end if
  ! Make sure that we capture the buoyancy period (for stratified flows)
  do n = 1, N_th
    Nmax = sqrt(abs(th_BC_Ymin_c1(n)))
    dt = min(dt, 0.1 * 2.d0 * pi / Nmax)
  end do

  ! Make sure we capture the Geostrophic Flow-through time
  !   because the split advection component may still be at most Vg !
  !dt=min(dt,min(dx(1),dy(1))/(LY*delta/Ro_inv*dTHdX))

  ! Use the model velocity to calculate the CFL number

  if (flavor == 'Front') then
    ! Add thermal wind to velocity when calculating the CFL number
    do n = 1, N_th
      do j = jstart, jend
        do k = 0, Nzp - 1
          do i = 0, Nxm1
            dt_x = CFL * dx(i) / abs(u1(i, k, j) - dTHdZ(n) &
                                     * gyf(j) / (Ro_inv / delta))
            dt_y = CFL * dy(j) / abs(u2(i, k, j))
            dt_z = CFL * dz(k) / abs(u3(i, k, j) + (1.d0 / (Ro_inv / delta)) &
                                     * dTHdX(n) * gyf(j))
            dt = min(dt, dt_x, dt_y, dt_z)
          end do
        end do
      end do
    end do
  else
    do j = 1, Nyp
      do k = 0, Nzp - 1
        do i = 0, Nxm1
          dt_x = CFL * dx(i) / abs(u1(i, k, j))
          dt_y = CFL * dy(j) / abs(u2(i, k, j))
          dt_z = CFL * dz(k) / abs(u3(i, k, j))
          dt = min(dt, dt_x, dt_y, dt_z)
        end do
      end do
    end do
  end if

  if (use_mpi) then
    call get_minimum_mpi(dt)
  end if

  if (dt <= 0) then
    if (rank == 0) &
      write (*, '("Error: dt <= 0 in courant")')
    ! Set DELTA_T to some small default value
    delta_t = 0.00001d0
  else if (dt >= 999.) then
    write (*, '("Warning: dt > 999, value capped at 999")')
    delta_t = 999.d0
  else
    delta_t = dt
  end if

  if (time + delta_t > save_stats_time) then
    delta_t = save_stats_time - time + 1.d-14
  else if (time + delta_t > save_flow_time) then
    delta_t = save_flow_time - time + 1.d-14
  else if (time + delta_t > time_limit) then
    delta_t = time_limit - time
  end if

  h_bar(1) = delta_t * (8.d0 / 15.d0)
  h_bar(2) = delta_t * (2.d0 / 15.d0)
  h_bar(3) = delta_t * (5.d0 / 15.d0)

  return
end
