! This file contains subroutines for inputting and outputting data in
! Diablo as well as all subroutines called directly from diablo.f90
!----*|--.---------.---------.---------.---------.---------.---------.-|-------|
subroutine initialize
  !----*|--.---------.---------.---------.---------.---------.---------.-|-------|
  include 'header'

  real version, current_version
  logical reset_time
  integer i, j, k, n
  real(rkind) Re

  open (11, file='input.dat', form='formatted', status='old')

  ! Read input file.
  !   (Note - if you change the following section of code, update the
  !    CURRENT_VERSION number to make obsolete previous input files !)

  current_version = 3.3
  read (11, *)
  read (11, *)
  read (11, *)
  read (11, *)
  read (11, *) flavor, version
  if (version /= current_version) stop 'Wrong input data format.'
  read (11, *)
  read (11, *) use_mpi, LES
  read (11, *)
  read (11, *) Re, beta, Lx, Lz, Ly
  nu = 1.d0 / Re
  read (11, *)
  read (11, *) nu_v_scale
  read (11, *)
  read (11, *) num_per_dir, create_new_flow
  read (11, *)
  read (11, *) wall_time_limit, time_limit, delta_t, reset_time, &
    variable_dt, CFL, update_dt
  read (11, *)
  read (11, *) verbosity, save_flow_dt, save_stats_dt, save_movie_dt, XcMovie, ZcMovie, YcMovie
  read (11, *)
  ! Read in the parameters for the N_th scalars
  do n = 1, N_th
    read (11, *)
    read (11, *) create_new_th(n)
    read (11, *)
    read (11, *) filter_th(n), filter_int(n)
    read (11, *)
    read (11, *) Ri(n), Pr(n)
  end do

  ! If we are using MPI, then Initialize the MPI Variables
  if (use_mpi) then
    call init_mpi
  else
    rank = 0
    rankY = 0
    rankZ = 0
  end if

  ! Check compatibility
  if ((use_mpi) .and. (num_per_dir == 3)) then
    write (*, *) 'Error: periodic.f90 is not parallelized in MPI'

  end if

  if (rank == 0) then
    write (*, *)
    write (*, *) '             ****** WELCOME TO DIABLO ******'
    write (*, *)
  end if

  ! Initialize case-specific packages.
  if (num_per_dir == 3) then
    call input_per
    call create_grid_per
    call init_per
  elseif (num_per_dir == 2) then
    call input_chan
    call create_grid_chan
    if (use_mpi) then
      call init_chan_mpi
    else
      call init_chan
    end if
    if (save_movie_dt /= 0) then
      call init_chan_movie
    end if
  elseif (num_per_dir == 1) then
    stop 'Error: Duct not implemented yet!'
  elseif (num_per_dir == 0) then
    stop 'Error: Cavity not implemented yet!'
  end if

  ! Initialize grid
  if (rank == 0) then
    write (*, '("Flavor:  ", A35)') flavor
    write (*, '("Nx    =  ", I10)') Nx
    write (*, '("Nyp   =  ", I10)') Nz
    write (*, '("Nz(p) =  ", I10)') Nyp
    do n = 1, N_th
      write (*, '("Scalar Number: ", I2)') n
      write (*, '("  Richardson number = ", ES12.5)') Ri(n)
      write (*, '("  Prandtl number    = ", ES12.5)') Pr(n)
    end do
    write (*, '("Nu   = ", ES12.5)') nu
    write (*, '("Beta = ", ES12.5)') beta
  end if

  ! Initialize FFT package (includes defining the wavenumber vectors).
  call init_fft

  ! Initialize RKW3 parameters.
  h_bar(1) = delta_t * (8.d0 / 15.d0)
  h_bar(2) = delta_t * (2.d0 / 15.d0)
  h_bar(3) = delta_t * (5.d0 / 15.d0)
  beta_bar(1) = 1.d0
  beta_bar(2) = 25.d0 / 8.d0
  beta_bar(3) = 9.d0 / 4.d0
  zeta_bar(1) = 0.d0
  zeta_bar(2) = -17.d0 / 8.d0
  zeta_bar(3) = -5.d0 / 4.d0

  ! Initialize values for reading of scalars
  num_read_th = 0
  do n = 1, N_th
    if (create_new_th(n)) then
      num_read_th = num_read_th
    else
      num_read_th = num_read_th + 1
      read_th_index(num_read_th) = n
    end if
  end do
  if (num_per_dir == 2) then
    call create_th_chan
  else if (num_per_dir == 3) then
    call create_th_per
  end if

  ! Create flow.
  if (create_new_flow) then
    if (num_per_dir == 3) then
      call create_flow_per
    elseif (num_per_dir == 2) then
      call create_flow_chan
    elseif (num_per_dir == 1) then
      stop 'Error: Duct not implemented yet!'
    elseif (num_per_dir == 0) then
      stop 'Error: Cavity not implemented yet!'
    end if
    if (rank == 0) &
      write (*, '("A new flowfield has been created.")')
  else
    if (rank == 0) &
      write (*, '("Reading flow...")')
    call read_flow
    if (rank == 0) &
      write (*, '("Starting at time step ", I10)') time_step

    if (use_mpi) then
      call ghost_chan_mpi
    end if

  end if

  ! Initialize flow.
  if (reset_time .or. create_new_flow) then
    save_flow_time = save_flow_dt
    save_stats_time = save_stats_dt
    save_movie_time = save_movie_dt
    time_step = 0
    time = 0

    call save_stats(save_movie_dt/=0,.false.)
    if (LES) call save_stats_LES_OOL(.true.)
  end if

  previous_time_step = time_step
  call wall_time(previous_wall_time)

  if (rank == 0) then
    write (*, *)
    write (*, *) '             ****** Done Initialising ******'
    write (*, *)
  end if


  return
end

!----*|--.---------.---------.---------.---------.---------.---------.-|-------|
subroutine save_stats(movie,final)
  !----*|--.---------.---------.---------.---------.---------.---------.-|-------|
  include 'header'
  logical movie, final
  real(rkind) wall_begin

  if (num_per_dir == 3) then
    call save_stats_per(final)
  elseif (num_per_dir == 2) then

    call wall_time(wall_begin)
    call save_stats_chan(movie,final)
    call wall_time(end_wall_time)
    if (rank == 0) &
      write (*,'("Elapsed Wall Time: ", ES12.3)') (end_wall_time - wall_begin)

    if (movie .and. Nz > 1) then
      call wall_time(wall_begin)
      call save_stats_mean_xy_chan
      call wall_time(end_wall_time)
      if (rank == 0) &
        write (*,'("Elapsed Wall Time: ", ES12.3)') (end_wall_time - wall_begin)
    end if

  end if

  return
end

!----*|--.---------.---------.---------.---------.---------.---------.-|-------|
subroutine read_flow
  !----*|--.---------.---------.---------.---------.---------.---------.-|-------|
  include 'header'
  character(len=55) fname
  character(len=55) fname_th(N_th)
  integer i, j, k, n, num_per_dir_t

  fname = 'start.h5'
  if (rank == 0) &
    write (*, '("Reading flow from " A10)') fname

  if (num_per_dir /= 2) then
    if (rank == 0) &
      write (*, '("Reading from HDF5 only implemented for Channel.")')
    stop
  end if
  call mpi_barrier(mpi_comm_world, ierror)
  call ReadHDF5(fname)

  ! Apply initial boundary conditions, set ghost cells
  if (use_mpi) then
    call apply_BC_vel_mpi
  else
    call apply_BC_vel_lower
    call apply_BC_vel_upper
  end if

  return
end

!----*|--.---------.---------.---------.---------.---------.---------.-|-------|
subroutine save_flow(final)
  !----*|--.---------.---------.---------.---------.---------.---------.-|-------|
  include 'header'
  character(len=55) fname
  character(len=55) fname_th(N_th)
  integer i, j, k, n
  logical final, flage, save_pressure
  real(rkind) wall_begin

  call wall_time(wall_begin)

  if (rank == 0) &
    write (*, '("Saving output file.")')

  save_pressure = .false.
  if (final) then
    fname = 'end.h5'
    save_pressure = .true.
  else
    write (fname,'(A4, I0.6, A3)') 'out.', time_step, '.h5'
  end if

  if (num_per_dir /= 2) then
    if (rank == 0) &
      write (*, '("Saving to HDF5 only implemented for Channel.")')
    stop
  end if
  call mpi_barrier(mpi_comm_world, ierror)
  call WriteHDF5(fname, save_pressure)

  call wall_time(end_wall_time)
  if (rank == 0) &
    write (*,'("Elapsed Time: ", ES12.5)') (end_wall_time - wall_begin)

  return
end

!----*|--.---------.---------.---------.---------.---------.---------.-|-------|
subroutine end_run(flag)
  !----*|--.---------.---------.---------.---------.---------.---------.-|-------|

  include 'header'

  logical flag, file_exists

  flag = .false.
  ! Check for the time
  call wall_time(end_wall_time)
  if (end_wall_time - start_wall_time > wall_time_limit) then
    if (rank == 0) &
      write (*, '("STOP because of wall-time hit!")')
    flag = .true.
  end if

  if (time >= time_limit) then
    if (rank == 0) &
      write (*, '("STOP because of simulation end-time hit!")')
    flag = .true.
  end if

  inquire (file="stop.now", exist=file_exists)
  if (file_exists) then
    if (rank == 0) &
      write (*, '("STOP because of stop.now file!")')
    flag = .true.
  end if

  return
end

!----*|--.---------.---------.---------.---------.---------.---------.-|-------|
subroutine wall_time(wt)
  !----*|--.---------.---------.---------.---------.---------.---------.-|-------|
  !
  ! Return wall-clock time as seconds after Jan. 1, 2016.
  ! Support for leap year is not included anymore.
  !
  ! By using a 'save' statement, the wall-time after the first
  ! call to the subroutine could be computed, but that is not
  ! intended with the present subroutine (e.g. the history file)
  !
  implicit none

  real(kind(0.d0)) wt
  integer val(8), i, shift, day

  integer mon(12, 2)
  data mon/ &
    31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31, &
    31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31/
  !
  ! Get current date and time
  ! val(1) : year
  ! val(2) : month
  ! val(3) : day
  ! val(4) : difference to GMT
  ! val(5) : hour
  ! val(6) : minute
  ! val(7) : second
  ! val(8) : 1/1000 second
  !
  call date_and_time(values=val)
  !
  ! Determine leap year
  !
  if (mod(val(1), 4) == 0) then
    if (mod(val(1), 100) == 0) then
      if (mod(val(1), 400) == 0) then
        shift = 2
      else
        shift = 1
      end if
    else
      shift = 2
    end if
  else
    shift = 1
  end if
  !
  ! Construct day of the year
  !
  day = val(3) - 1
  do i = 1, val(2) - 1
    day = day + mon(i, shift)
  end do
  !
  ! And compute wall-clock time
  !
  wt = (val(1) - 2016) * 365 * 86400 + &
       day * 86400 + val(5) * 3600 + val(6) * 60 + val(7) + dble(val(8) / 1000.d0)

end
