!----*|--.---------.---------.---------.---------.---------.---------.-|-------|
subroutine filter_chan(n)
  !----*|--.---------.---------.---------.---------.---------.---------.-|-------|
  ! This subroutine applies a filter to the highest wavenumbers
  ! It should be applied to the scalar in Fourier space
  ! The filter used is a sharpened raised cosine filter in the horizontal
  ! and a fourth order implicit compact filter in the vertical, with the
  ! parameter alpha determining the width of the vertical filtering window

  include 'header'

  integer i, j, k, js, je, n

  ! Variables for horizontal filtering
  real(rkind) sigma(0:Nxp - 1, 0:twoNkz), sigma0

  ! Variables for vertical filtering
  real(rkind), parameter :: alpha = 0.d0

  ! Parameters for a larger stencil filter
  real(rkind) f_a, f_b, f_c

  ! Set the filtering constants for the horizontal direction
  do i = 0, Nxp - 1
    do k = 0, twoNkz
      sigma0 = 0.5d0 * (1.d0 + &
                        cos(sqrt((kx(i) * Lx * 1.d0 / float(Nx))**2.d0 &
                                 + (kz(k) * Lz * 1.d0 / float(Nz))**2.d0)))
      ! Apply a sharpened raised cosine filter
      sigma(i, k) = sigma0**4.d0 * (35.d0 - 84.d0 * sigma0 &
                                    + 70.d0 * sigma0**2.d0 - 20.d0 * sigma0**3.d0)
    end do
  end do

  ! Do the spectral filtering in the horizontal
  do k = 0, twoNkz
    do i = 0, Nxp - 1
      do j = jstart_th(n), jend_th(n)
        cth(i, k, j, n) = cth(i, k, j, n) * sigma(i, k)
      end do
    end do
  end do

  ! Filter the passive scalar, TH in the vertical direction
  ! Set the filtering constants
  !      f_a=(1.d0/8.d0)*(5.d0+6.d0*alpha)
  !      f_b=0.5d0*(1.d0+2.d0*alpha)
  !      f_c=(-1.d0/8.d0)*(1.d0-2.d0*alpha)
  ! First, zero the tridiagonal matrix components
  !      DO I=0,Nkx
  !        DO J=1,Nyp
  !          MATD_C(I,J)=1.d0
  !          MATL_C(I,J)=0.d0
  !          MATU_C(I,J)=0.d0
  !          VEC_C(I,J)=0.d0
  !        END DO
  !      END DO
  !      DO K=1,twoNkz
  !        DO I=1,Nkx
  ! Construct the centered difference terms
  !          DO J=2,Nyp-1
  !            MATL_C(I,J)=alpha
  !            MATD_C(I,J)=1.d0
  !            MATU_C(I,J)=alpha
  !            VEC_C(I,J)=f_a*CTH(I,K,J,n)
  !     &                +(f_b/2.d0)*(CTH(I,K,J+1,n)+CTH(I,K,J-1,n))
  !     &                +(f_c/2.d0)*(CTH(I,K,J+2,n)+CTH(I,K,J-2,n))
  !          END DO
  ! Now, construct the equations for the boundary nodes
  !          J=1
  !            MATL_C(I,J)=0.d0
  !            MATD_C(I,J)=1.d0
  !            MATU_C(I,J)=0.d0
  !            VEC_C(I,J)=CTH(I,K,J,n)
  !          J=Nyp
  !            MATL_C(I,J)=0.d0
  !            MATD_C(I,J)=1.d0
  !            MATU_C(I,J)=0.d0
  !            VEC_C(I,J)=CTH(I,K,J,n)
  !         END DO
  ! Now, solve the tridiagonal system
  !         CALL THOMAS_COMPLEX(MATL_C,MATD_C,MATU_C,VEC_C,Nyp,Nkx)
  !         DO I=1,Nkx
  !           DO J=JSTART_TH(N),JEND_TH(N)
  !             CTH(I,K,J,n)=VEC_C(I,J)
  !           END DO
  !         END DO
  ! END DO K
  !       END DO

  return
end
