!----*|--.---------.---------.---------.---------.---------.---------.-|-------|
subroutine WriteHDF5(fname, save_pressure)
  !----*|--.---------.---------.---------.---------.---------.---------.-|-------|
  use hdf5

  include 'header'

  character(len=55) fname
  logical final, save_pressure

  real(rkind) tmp(Nx, Nyp, Nz)

  ! Dataset names
  character(len=10) :: dname

  ! Identifiers
  integer(hid_t) :: file_id, dset_id
  integer(hid_t) :: filspace_id, memspace_id

  ! Identifiers
  integer(hid_t) :: gid, selspace_id
  integer(hid_t) :: plist_id_d

  ! Dimensions in the memory and in the file
  integer(hsize_t), dimension(3) :: dimsm, dimsf

  integer(hsize_t), dimension(3) :: chunk_dims, count, offset
  integer(hsize_t), dimension(3) :: stride, block, offset_m

  integer :: rHDF5 = 3, arank = 1

  integer(hsize_t), dimension(1)       :: adims
  integer(hid_t)                      :: aid, tspace
  real, dimension(1)               :: treal(3)
  integer, dimension(1)               :: tint(3)
  character(len=80)                        :: namnbuf
  character(len=20)                        :: sttimec

  integer Error, ith

  double precision En(4)

  dimsm(1:3) = (/Nx, Nyp, Nz/)
  dimsf(1:3) = (/Nx, Nyp, Nz/)

  ! Flow fields are saved in the fractional grid. We use a basic
  ! interpolation
  !
  ! u_j+1/2=u_j+u_j+1
  !
  ! on the way back we just invert the relation in order to have
  ! exact values.

  write (*, *) 'Writing flow to ', fname

  chunk_dims(1) = Nx
  chunk_dims(2) = 1
  chunk_dims(3) = Nz

  block(1) = Nx
  block(3) = Nz

  ! Stride and count for number of rows and columns in each dimension
  stride = 1
  count = 1

  ! Offset determined by the rank of a processor
  offset(1) = 0
  offset(3) = 0

  block(2) = Nyp
  offset(2) = 0

  ! Initialize interface
  call h5open_f(Error)

  ! Create the file collectively
  call h5fcreate_f(trim(fname), h5f_acc_trunc_f, &
                   file_id, Error)

  arank = 1
  adims = 3
  call h5screate_simple_f(arank, adims, tspace, Error)

  ! -----------------------------
  ! Resolution
  call h5acreate_f(file_id, 'Resolution', h5t_std_i32le, tspace, &
                   aid, Error)
  tint(1) = Nx
  tint(2) = Nyp
  tint(3) = Nz
  call h5awrite_f(aid, h5t_native_integer, tint, adims, Error)
  call h5aclose_f(aid, Error)
  ! -----------------------------
  ! Close dataspace
  call h5sclose_f(tspace, Error)

  ! -----------------------------
  ! Date
  adims = 20
  call h5screate_simple_f(arank, adims, tspace, Error)

  call h5acreate_f(file_id, 'Date', h5t_c_s1, tspace, aid, &
                   Error)
  call time_string(sttimec)
  call h5awrite_f(aid, h5t_c_s1, sttimec, adims, Error)
  call h5aclose_f(aid, Error)
  call h5sclose_f(tspace, Error)
  ! -----------------------------

  ! -----------------------------
  ! Extra info
  adims = 80
  call h5screate_simple_f(arank, adims, tspace, Error)

  call h5acreate_f(file_id, 'Info', h5t_c_s1, tspace, aid, &
                   Error)
  namnbuf = ' (Put here what you want) '
  call h5awrite_f(aid, h5t_c_s1, namnbuf, adims, Error)
  call h5aclose_f(aid, Error)
  call h5sclose_f(tspace, Error)
  ! -----------------------------

  call h5screate_f(h5s_scalar_f, tspace, Error)


  ! Create the timestep group
  call h5gcreate_f(file_id, 'Timestep', gid, Error)

  ! -----------------------------
  ! Time
  call h5acreate_f(gid, 'Time', h5t_ieee_f64le, tspace, &
                   aid, Error)
  call h5awrite_f(aid, h5t_native_double, time, adims, Error)
  call h5aclose_f(aid, Error)
  ! ----------------------------

  call h5sclose_f(tspace, Error)

  ! Convert to physical space
  call fft_xz_to_physical(cu1, u1, 0, Nyp + 1)
  call fft_xz_to_physical(cu2, u2, 0, Nyp + 1)
  call fft_xz_to_physical(cu3, u3, 0, Nyp + 1)
  do ith = 1, N_th
    call fft_xz_to_physical(cth(0, 0, 0, ith), th(0, 0, 0, ith), 0, Nyp + 1)
  end do

  ! Create property list for the chunked dataset creation
  call h5pcreate_f(h5p_dataset_create_f, plist_id_d, Error)
  call h5pset_chunk_f(plist_id_d, rHDF5, chunk_dims, Error)

  ! Create the dataspace for ur
  call h5screate_simple_f(rHDF5, dimsf, filspace_id, Error)
  call h5screate_simple_f(rHDF5, dimsm, memspace_id, &
                          Error)

  do ith = 1, 3 + N_th
    ! Here it starts the loop--->

    select case (ith)
    case (1)
      call swapzy(u1, tmp)
      dname = "U"
    case (2)
      ! Interpolation to the fractional grid
      call g2gf(u2)
      call swapzy(u2, tmp)
      call gf2g(u2)
      dname = "V"
    case (3)
      call swapzy(u3, tmp)
      dname = "W"
    case (4:)
      call swapzy(th(0, 0, 0, ith - 3), tmp)
      dname = "TH"//char(ith + 45)
    end select

    call h5dcreate_f(gid, trim(dname), h5t_ieee_f64le, &
                     filspace_id, dset_id, Error, dcpl_id=plist_id_d)

    ! Select hyperslab in the file.
    ! call h5dget_space_f(dsetur_id, selspace_id, Error)
    call h5sselect_hyperslab_f(filspace_id, h5s_select_set_f, &
                               offset, count, Error, stride, block)

    offset_m(1:3) = 0
    call h5sselect_hyperslab_f(memspace_id, h5s_select_set_f, &
                               offset_m, count, Error, stride, block)

    ! Write the dataset collectively
    call h5dwrite_f(dset_id, h5t_native_double, &
                    tmp, &
                    dimsm, Error, file_space_id=filspace_id, &
                    mem_space_id=memspace_id)

    ! Close dateset
    call h5dclose_f(dset_id, Error)
  end do

  ! In the case of saving for the pressure as well
  if (save_pressure) then
    call fft_xz_to_physical(cp, p, 0, Nyp + 1)

    call swapzy(p, tmp)
    dname = "P"

    call h5dcreate_f(gid, trim(dname), h5t_ieee_f64le, &
                     filspace_id, dset_id, Error, dcpl_id=plist_id_d)

    ! Select hyperslab in the file.
    ! call h5dget_space_f(dsetur_id, selspace_id, Error)
    call h5sselect_hyperslab_f(filspace_id, h5s_select_set_f, &
                               offset, count, Error, stride, block)

    call h5sselect_hyperslab_f(memspace_id, h5s_select_set_f, &
                               offset_m, count, Error, stride, block)

    ! Write the dataset collectively
    call h5dwrite_f(dset_id, h5t_native_double, &
                    tmp, &
                    dimsm, Error, file_space_id=filspace_id, &
                    mem_space_id=memspace_id)

    ! Close dateset
    call h5dclose_f(dset_id, Error)
    call fft_xz_to_fourier(p, cp, 0, Nyp + 1)
  end if

  ! Close the dataspace for the memory and for the file
  call h5sclose_f(filspace_id, Error)
  call h5sclose_f(memspace_id, Error)

  ! Close the properties for the dataspace creation and the writing
  call h5pclose_f(plist_id_d, Error)

  ! Close groups
  call h5gclose_f(gid, Error)
  call h5fclose_f(file_id, Error)
  call h5close_f(Error)

  call fft_xz_to_fourier(u1, cu1, 0, Nyp + 1)
  call fft_xz_to_fourier(u2, cu2, 0, Nyp + 1)
  call fft_xz_to_fourier(u3, cu3, 0, Nyp + 1)
  do ith = 1, N_th
    call fft_xz_to_fourier(th(0, 0, 0, ith), cth(0, 0, 0, ith), 0, Nyp + 1)
  end do

  ! call mpi_finalize(ierror)
  ! stop

end subroutine WriteHDF5

!----*|--.---------.---------.---------.---------.---------.---------.-|-------|
subroutine ReadHDF5(fname)
  !----*|--.---------.---------.---------.---------.---------.---------.-|-------|
  use hdf5

  include 'header'

  character(len=55) fname
  logical final, read_pressure

  real(rkind) tmp(Nx, Nyp, Nz)
  !
  ! HDF5 ------------------------------------------------------
  !
  ! Dataset names
  character(len=10) :: dname

  ! Identifiers
  integer(hid_t) :: file_id, dset_id
  integer(hid_t) :: filspace_id, memspace_id

  ! Identifiers
  integer(hid_t) :: gid, selspace_id

  ! Dimensions in the memory and in the file
  integer(hsize_t), dimension(3) :: dimsm, dimsf

  integer(hsize_t), dimension(3) :: count, offset
  integer(hsize_t), dimension(3) :: stride, block, offset_m

  integer :: rHDF5 = 3

  integer(hsize_t), dimension(1)       :: adims
  integer(hid_t)                      :: aid, tspace
  real, dimension(1)               :: treal(3)
  integer, dimension(1)               :: tint(3)
  character(len=80)                        :: namnbuf
  character(len=20)                        :: sttimec

  integer Error, ith

  double precision En(4)

  dimsm(1:3) = (/Nx, Nyp, Nz/)
  dimsf(1:3) = (/Nx, Nyp, Nz/)

  ! Flow fields are saved in the fractional grid. We use a basic
  ! interpolation
  !
  ! u_j+1/2=u_j+u_j+1
  !
  ! on the way back we just invert the relation in order to have
  ! exact values.

  block(1) = Nx
  block(3) = Nz

  ! Stride and count for number of rows and columns in each dimension
  stride = 1
  count = 1

  ! Offset determined by the rank of a processor
  offset(1) = 0
  offset(3) = 0

  block(2) = Nyp
  offset(2) = 0

  ! Initialize interface
  call h5open_f(Error)

  ! Create the file collectively
  call h5fopen_f(trim(fname), h5f_acc_rdonly_f, &
                 file_id, Error)

  adims = 3

  ! -----------------------------
  ! Resolution
  ! -----------------------------
  call h5aopen_by_name_f(file_id, '.', 'Resolution', aid, Error)
  call h5aread_f(aid, h5t_native_integer, tint, adims, Error)
  call h5aclose_f(aid, Error)
  ! Check that the resolution is of the same kind
  if ((tint(1) /= Nx) .or. &
      (tint(2) /= Nyp) .or. &
      (tint(3) /= Nz)) then
    if (rank == 0) then
      write (*, *) ' Error. File and program have ', &
        'different resolutions. '
      write (*, *) ' Program: ', Nx, Nyp, Nz
      write (*, *) ' File   : ', tint(1:3)
    end if
    stop
  end if

  call h5gopen_f(file_id, "/Timestep", gid, Error)
  ! -----------------------------
  ! Time stamps
  ! -----------------------------
  call h5aopen_by_name_f(gid, '.', 'Time', aid, Error)
  call h5aread_f(aid, h5t_native_double, time, adims, Error)
  call h5aclose_f(aid, Error)
  ! -----------------------------

  ! Dataspace in memory
  call h5screate_simple_f(rHDF5, dimsm, memspace_id, &
                          Error)

  do ith = 1, 3 + N_th
    ! Here it starts the loop--->
    select case (ith)
    case (1)
      dname = "U"
    case (2)
      dname = "V"
    case (3)
      dname = "W"
    case (4:)
      dname = "TH"//char(ith + 45)
    end select

    call h5dopen_f(gid, trim(dname), dset_id, Error)
    call h5dget_space_f(dset_id, filspace_id, Error)

    ! Select hyperslab in the file.
    call h5sselect_hyperslab_f(filspace_id, h5s_select_set_f, &
                               offset, count, Error, stride, block)

    offset_m(1:3) = 0
    call h5sselect_hyperslab_f(memspace_id, h5s_select_set_f, &
                               offset_m, count, Error, stride, block)

    ! Write the dataset collectively
    call h5dread_f(dset_id, h5t_native_double, &
                   tmp, &
                   dimsm, Error, file_space_id=filspace_id, &
                   mem_space_id=memspace_id)

    select case (ith)
    case (1)
      call swapyz(tmp, u1)
    case (2)
      call swapyz(tmp, u2)
      ! Interpolation to the collocated grid
      call gf2g(u2)
    case (3)
      call swapyz(tmp, u3)
    case (4:)
      call swapyz(tmp, th(0, 0, 0, ith - 3))
    end select

    ! Close dateset
    call h5sclose_f(filspace_id, Error)
    call h5dclose_f(dset_id, Error)
  end do

  ! Decide whether to compute the pressure or to read
  call h5lexists_f(gid, 'P', read_pressure, Error)
  if (read_pressure) then
    dname = "P"
    call h5dopen_f(gid, trim(dname), dset_id, Error)
    call h5dget_space_f(dset_id, filspace_id, Error)

    ! Select hyperslab in the file.
    call h5sselect_hyperslab_f(filspace_id, h5s_select_set_f, &
                               offset, count, Error, stride, block)

    offset_m(1:3) = 0
    call h5sselect_hyperslab_f(memspace_id, h5s_select_set_f, &
                               offset_m, count, Error, stride, block)

    ! Write the dataset collectively
    call h5dread_f(dset_id, h5t_native_double, &
                   tmp, &
                   dimsm, Error, file_space_id=filspace_id, &
                   mem_space_id=memspace_id) !, xfer_prp = plist_id_w)

    call swapyz(tmp, p)
    ! Close dateset
    call h5sclose_f(filspace_id, Error)
    call h5dclose_f(dset_id, Error)

    call fft_xz_to_fourier(p, cp, 0, Nyp + 1)
  end if

  ! Close the dataspace for the memory
  call h5sclose_f(memspace_id, Error)

  ! Close groups
  call h5gclose_f(gid, Error)
  call h5fclose_f(file_id, Error)
  call h5close_f(Error)

  if (variable_dt) then
    call courant
  end if

  ! Convert to physical space
  call fft_xz_to_fourier(u1, cu1, 0, Nyp + 1)
  call fft_xz_to_fourier(u2, cu2, 0, Nyp + 1)
  call fft_xz_to_fourier(u3, cu3, 0, Nyp + 1)
  do ith = 1, N_th
    call fft_xz_to_fourier(th(0, 0, 0, ith), cth(0, 0, 0, ith), 0, Nyp + 1)
  end do

  if (.not. read_pressure) then
    if (num_per_dir == 3) then
      ! CALL POISSON_P_PER
      call rem_div_per
    elseif (num_per_dir == 2) then
      call poisson_p_chan
    end if
  end if

end subroutine ReadHDF5

!----*|--.---------.---------.---------.---------.---------.---------.-|-------|
subroutine ReadGridHDF5(fname, coord)
  !----*|--.---------.---------.---------.---------.---------.---------.-|-------|
  use hdf5

  implicit none

  include 'header'

  character(len=55) fname

  ! Identifiers
  integer(hid_t) :: file_id, dset_id
  integer(hid_t) :: filspace_id, memspace_id
  ! Identifiers
  integer(hid_t) :: gid, selspace_id

  ! Dataset names
  integer           :: coord
  character(len=10) :: cname
  ! Dimensions in the memory and in the file
  integer(hsize_t), dimension(1)  :: dimsm, dimsf
  integer(hsize_t), dimension(1)  :: idims, imaxd

  integer(hsize_t), dimension(1)  :: count, offset
  integer(hsize_t), dimension(1)  :: stride, block, offset_m

  integer           :: Error, rHDF5, ith

  ! Initialize interface
  call h5open_f(Error)

  ! Create the file collectively
  call h5fopen_f(trim(fname), h5f_acc_rdonly_f, &
                 file_id, Error)


  select case (coord)
  case (1)
    write (*, *) ' Error 235454. Not implemented yet   ! '
    stop
  case (2)
    cname = 'y'

    dimsm = Nyp + 2
    dimsf = Nyp

    ! Stride and count for number of rows and columns in each dimension
    stride = 1
    count = 1

    ! Offset determined by the rank of a processor
    block = Nyp + 1
    offset = 0
  case (3)
    write (*, *) ' Error 235455. Not implemented yet   ! '
    stop
  end select

  call h5gopen_f(file_id, "/grids", gid, Error)

  ! Dataspace in memory
  rHDF5 = 1
  call h5screate_simple_f(rHDF5, dimsm, memspace_id, &
                          Error)

  call h5dopen_f(gid, trim(cname), dset_id, Error)
  call h5dget_space_f(dset_id, filspace_id, Error)

  ! Check for the dimensions
  call h5sget_simple_extent_dims_f(filspace_id, idims, imaxd, Error)
  if (idims(1) - 1 /= dimsf(1)) then
    write (*, *) ' Grid file and program do not match. '
    write (*, *) '   gridfile (', trim(cname), '): ', dimsf
    write (*, *) '   program     : ', idims - 1
    stop
  end if

  ! Select hyperslab in the file.
  call h5sselect_hyperslab_f(filspace_id, h5s_select_set_f, &
                             offset, count, Error, stride, block)

  offset_m = 1
  call h5sselect_hyperslab_f(memspace_id, h5s_select_set_f, &
                             offset_m, count, Error, stride, block)

  ! Write the dataset collectively
  call h5dread_f(dset_id, h5t_native_double, &
                 gy, &
                 dimsm, Error, file_space_id=filspace_id, &
                 mem_space_id=memspace_id)

  ! Close dateset
  call h5sclose_f(filspace_id, Error)
  call h5dclose_f(dset_id, Error)

  ! Close the dataspace for the memory
  call h5sclose_f(memspace_id, Error)

  ! Close groups
  call h5gclose_f(gid, Error)
  call h5fclose_f(file_id, Error)
  call h5close_f(Error)

  ! Calculate the GYF in the interior
  do ith = 1, Nyp
    gyf(ith) = 0.5 * (gy(ith) + gy(ith + 1))
  end do

  ! ###############################
  !    Get the outer ghost cells
  ! ###############################

  gyf(0) = 2.0 * gyf(1) - gyf(2)
  gyf(Nyp + 1) = 2.d0 * gyf(Nyp) - gyf(Nyp - 1)

end subroutine

!----*|--.---------.---------.---------.---------.---------.---------.-|-------|
subroutine g2gf(var)
  !----*|--.---------.---------.---------.---------.---------.---------.-|-------|

  implicit none

  include 'header'

  real(rkind) var(0:Nx + 1, 0:Nz + 1, 0:Nyp + 1)
  integer x, z, y

  do x = 0, Nx - 1
    do z = 0, Nz - 1
      var(x, z, 1) = var(x, z, 2)
      var(x, z, Nyp + 1) = var(x, z, Nyp)
      do y = 1, Nyp
        var(x, z, y) = 0.5 * (var(x, z, y) + var(x, z, y + 1))
      end do
    end do
  end do

end subroutine

!----*|--.---------.---------.---------.---------.---------.---------.-|-------|
subroutine gf2g(var)
  !----*|--.---------.---------.---------.---------.---------.---------.-|-------|

  implicit none

  include 'header'

  real(rkind) var(0:Nx + 1, 0:Nz + 1, 0:Nyp + 1)
  integer x, z, y
  integer xzbox

  do x = 0, Nx - 1
    do z = 0, Nz - 1
      var(x, z, Nyp + 1) = var(x, z, Nyp)
    end do
  end do

  do x = 0, Nx - 1
    do z = 0, Nz - 1
      do y = Nyp, 1, -1
        var(x, z, y) = 2 * var(x, z, y) - var(x, z, y + 1)
      end do
    end do
  end do

  ! Impose the values at the boundary as prescribed in the
  ! code in order to have zero mass flux
  var(:, :, 1) = -var(:, :, 2)
  var(:, :, Nyp + 1) = -var(:, :, Nyp)

end subroutine

!----*|--.---------.---------.---------.---------.---------.---------.-|-------|
subroutine swapzy(in, out)
  !----*|--.---------.---------.---------.---------.---------.---------.-|-------|

  include 'header'

  real(rkind) in(0:Nx + 1, 0:Nz + 1, 0:Nyp + 1)
  real(rkind) out(1:Nx, 1:Nyp, 1:Nz)
  integer x, z, y

  out = 0.d0
  do x = 0, Nx - 1
    do y = 1, Nyp
      do z = 0, Nz - 1
        out(x + 1, y, z + 1) = in(x, z, y)
      end do
    end do
  end do

end subroutine

!----*|--.---------.---------.---------.---------.---------.---------.-|-------|
subroutine swapyz(in, out)
  !----*|--.---------.---------.---------.---------.---------.---------.-|-------|

  include 'header'

  real(rkind) out(0:Nx + 1, 0:Nz + 1, 0:Nyp + 1)
  real(rkind) in(1:Nx, 1:Nyp, 1:Nz)
  integer x, z, y

  out = 0.d0
  do x = 0, Nx - 1
    do y = 1, Nyp
      do z = 0, Nz - 1
        out(x, z, y) = in(x + 1, y, z + 1)
      end do
    end do
  end do

end subroutine

!----*|--.---------.---------.---------.---------.---------.---------.-|-------|
subroutine time_string(cdt)
  !----*|--.---------.---------.---------.---------.---------.---------.-|-------|
  !
  ! Construct string in the format '19-DEC-2005 22:47:06'
  !
  implicit none

  integer i

  integer val(8)
  character(len=20) cdt
  character(len=3) monc

  call date_and_time(values=val)

  if (val(2) == 1) then
    monc = 'JAN'
  else if (val(2) == 2) then
    monc = 'FEB'
  else if (val(2) == 3) then
    monc = 'MAR'
  else if (val(2) == 4) then
    monc = 'APR'
  else if (val(2) == 5) then
    monc = 'MAY'
  else if (val(2) == 6) then
    monc = 'JUN'
  else if (val(2) == 7) then
    monc = 'JUL'
  else if (val(2) == 8) then
    monc = 'AUG'
  else if (val(2) == 9) then
    monc = 'SEP'
  else if (val(2) == 10) then
    monc = 'OCT'
  else if (val(2) == 11) then
    monc = 'NOV'
  else if (val(2) == 12) then
    monc = 'DEC'
  else
    monc = 'XXX'
  end if

  write (cdt, '(i2,a1,a3,a1,i4,a1,i2,a1,i2,a1,i2)') &
    val(3), '-', monc, '-', val(1), ' ', val(5), ':', val(6), ':', val(7)
  do i = 1, 2
    if (cdt(i:i) == ' ') then
      cdt(i:i) = '0'
    end if
  end do
  do i = 13, 20
    if (cdt(i:i) == ' ') then
      cdt(i:i) = '0'
    end if
  end do

end subroutine time_string
