!----*|--.---------.---------.---------.---------.---------.---------.-|-------|

block data zbqlbd01
!
!       Initializes seed array etc. for random number generator.
!       The values below have themselves been generated using the
!       NAG generator.
!
common / zbql0001 / zbqlix, b, c, inx
double precision zbqlix(43), b, c
integer inx
data(zbqlix(inx), inx=1, 43) / 8.001441d7, 5.5321801d8, &
  1.69570999d8, 2.88589940d8, 2.91581871d8, 1.03842493d8, &
  7.9952507d7, 3.81202335d8, 3.11575334d8, 4.02878631d8, &
  2.49757109d8, 1.15192595d8, 2.10629619d8, 3.99952890d8, &
  4.12280521d8, 1.33873288d8, 7.1345525d7, 2.23467704d8, &
  2.82934796d8, 9.9756750d7, 1.68564303d8, 2.86817366d8, &
  1.14310713d8, 3.47045253d8, 9.3762426d7, 1.09670477d8, &
  3.20029657d8, 3.26369301d8, 9.441177d6, 3.53244738d8, &
  2.44771580d8, 1.59804337d8, 2.07319904d8, 3.37342907d8, &
  3.75423178d8, 7.0893571d7, 4.26059785d8, 3.95854390d8, &
  2.0081010d7, 5.9250059d7, 1.62176640d8, 3.20429173d8, &
  2.63576576d8 /
data b/4.294967291d9/
data c/0.0d0/
end
!*****************************************************************
!*****************************************************************
!*****************************************************************
subroutine zbqlini(seed)
  !*****************************************************************
  !       To initialize the random number generator - either
  !       repeatably or nonrepeatably. Need double precision
  !       variables because integer storage can't handle the
  !       numbers involved
  !*****************************************************************
  !        ARGUMENTS
  !        =========
  !        SEED        (integer, input). User-input number which generates
  !                elements of the array ZBQLIX, which is subsequently used
  !                in the random number generation algorithm. If SEED=0,
  !                the array is seeded using the system clock if the
  !                FORTRAN implementation allows it.
  !*****************************************************************
  !        PARAMETERS
  !        ==========
  !        LFLNO        (integer). Number of lowest file handle to try when
  !                opening a temporary file to copy the system clock into.
  !                Default is 80 to keep out of the way of any existing
  !                open files (although the program keeps searching till
  !                it finds an available handle). If this causes problems,
  !               (which will only happen if handles 80 through 99 are
  !               already in use), decrease the default value.
  !*****************************************************************
  integer lflno
  character(len=10) name
  character(len=30) command
  parameter(lflno=80)
  !*****************************************************************
  !        VARIABLES
  !        =========
  !        SEED        See above
  !        ZBQLIX        Seed array for the random number generator. Defined
  !                in ZBQLBD01
  !        B,C        Used in congruential initialisation of ZBQLIX
  !        SS,MM,}        System clock secs, mins, hours and days
  !        HH,DD }
  !        FILNO        File handle used for temporary file
  !        INIT        Indicates whether generator has already been initialised
  !
  integer seed, ss, mm, hh, dd, filno, i
  integer init, inx
  double precision zbqlix(43), b, c
  double precision tmpvar1, dss, dmm, dhh, ddd

  common / zbql0001 / zbqlix, b, c, inx
  save init

  !
  !        Ensure we don't call this more than once in a program
  !
  if (init >= 1) then
    if (init == 1) then
      write (*, 1)
      init = 2
    endif
    return
  else
    init = 1
  endif
  !
  !       If SEED = 0, cat the contents of the clock into a file
  !       and transform to obtain ZQBLIX(1), then use a congr.
  !       algorithm to set remaining elements. Otherwise take
  !       specified value of SEED.
  !
  !>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
  !>>>>>>>        NB FOR SYSTEMS WHICH DO NOT SUPPORT THE  >>>>>>>
  !>>>>>>>        (NON-STANDARD) 'CALL SYSTEM' COMMAND,    >>>>>>>
  !>>>>>>>        THIS WILL NOT WORK, AND THE FIRST CLAUSE >>>>>>>
  !>>>>>>>        OF THE FOLLOWING IF BLOCK SHOULD BE         >>>>>>>
  !>>>>>>>        COMMENTED OUT.                                 >>>>>>>
  !>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
  if (seed >= 0) then
    !>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
    !>>>>>>>        COMMENT OUT FROM HERE IF YOU DON'T HAVE  >>>>>>>
    !>>>>>>>        'CALL SYSTEM' CAPABILITY ...                 >>>>>>>
    !>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
    if (seed < 10) then
      write (name, '(A,I1,A)') 'zbql', seed, '.tmp'
    else
      write (name, '(A,I2,A)') 'zbql', seed, '.tmp'
    end if
    command = 'date +%S%M%H%j > '//name
    call system(command)
    !
    !       Try all file numbers for LFLNO to 999
    !
    filno = lflno
10  open (filno, file=name, err=11)
    goto 12
11  filno = filno + 1
    if (filno > 999) then
      write (*, 2)
      return
    endif
    goto 10
12  read (filno, '(3(I2),I3)') ss, mm, hh, dd
    ss = ss + seed
    close (filno)
    command = 'rm '//name
    call system(command)
    dss = dint((dble(ss) / 6.0d1) * b)
    dmm = dint((dble(mm) / 6.0d1) * b)
    dhh = dint((dble(hh) / 2.4d1) * b)
    ddd = dint((dble(dd) / 3.65d2) * b)
    tmpvar1 = dmod(dss + dmm + dhh + ddd, b)
    !<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
    !<<<<<<<<        ... TO HERE (END OF COMMENTING OUT FOR           <<<<<<<
    !<<<<<<<<        USERS WITHOUT 'CALL SYSTEM' CAPABILITY          <<<<<<<
    !<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  else
    tmpvar1 = dmod(dble(seed), b)
  endif
  zbqlix(1) = tmpvar1
  do 100 i = 2, 43
    tmpvar1 = zbqlix(i - 1) * 3.0269d4
    tmpvar1 = dmod(tmpvar1, b)
    zbqlix(i) = tmpvar1
100 continue

1   format(//5x, '****Warning**** You have called routine ZBQLINI ', &
            'more than', /5x, 'once. I''m ignoring any subsequent calls.', //)
2   format(//5x, '**** Error **** In routine ZBQLINI, I couldn''t', &
            ' find an', /5x, &
            'available file number. To rectify the problem, decrease the ', &
            'value of', /5x, &
            'the parameter LFLNO at the start of this routine (in file ', &
            'randgen.f)', /5x, &
            'and recompile. Any number less than 100 should work.')
  end
  !*****************************************************************
  function zbqlu01(dummy)
    !
    !       Returns a uniform random number between 0 & 1, using
    !       a Marsaglia-Zaman type subtract-with-borrow generator.
    !       Uses double precision, rather than integer, arithmetic
    !       throughout because MZ's integer constants overflow
    !       32-bit integer storage (which goes from -2^31 to 2^31).
    !       Ideally, we would explicitly truncate all integer
    !       quantities at each stage to ensure that the double
    !       precision representations do not accumulate approximation
    !       Error; however, on some machines the use of DNINT to
    !       accomplish this is *seriously* slow (run-time increased
    !       by a factor of about 3). This double precision version
    !       has been tested against an integer implementation that
    !       uses long integers (non-standard and, again, slow) -
    !       the output was identical up to the 16th decimal place
    !       after 10^10 calls, so we're probably OK ...
    !
    double precision zbqlu01, dummy, b, c, zbqlix(43), x, b2, binv
    integer curpos, id22, id43, inx

    common / zbql0001 / zbqlix, b, c, inx
    save / zbql0001 /
    save curpos, id22, id43
    data curpos, id22, id43/1, 22, 43/

    b2 = b
    binv = 1.0d0 / b
5   x = zbqlix(id22) - zbqlix(id43) - c
    if (x < 0.0d0) then
      x = x + b
      c = 1.0d0
    else
      c = 0.0d0
    endif
    zbqlix(id43) = x
    !
    !     Update array pointers. Do explicit check for bounds of each to
    !     avoid expense of modular arithmetic. If one of them is 0 the others
    !     won't be
    !
    curpos = curpos - 1
    id22 = id22 - 1
    id43 = id43 - 1
    if (curpos == 0) then
      curpos = 43
    elseif (id22 == 0) then
      id22 = 43
    elseif (id43 == 0) then
      id43 = 43
    endif
    !
    !     The integer arithmetic there can yield X=0, which can cause
    !     problems in subsequent routines (e.g. ZBQLEXP). The problem
    !     is simply that X is discrete whereas U is supposed to
    !     be continuous - hence if X is 0, go back and generate another
    !     X and return X/B^2 (etc.), which will be uniform on (0,1/B).
    !
    if (x < binv) then
      b2 = b2 * b
      goto 5
    endif

    zbqlu01 = x / b2

  end
  !*****************************************************************
  function zbqlnor(mu, sigma)
    !
    !       Returns a random number Normally distributed with mean
    !       MU and standard deviation |SIGMA|, using the Box-Muller
    !       algorithm
    !
    double precision theta, r, zbqlnor, zbqlu01, pi, mu, sigma
    double precision spare
    integer status
    save status, spare, pi
    data status/-1/

    if (status == -1) pi = 4.0d0 * datan(1.0d0)

    if (status <= 0) then
      theta = 2.0d0 * pi * zbqlu01(0.0d0)
      r = dsqrt(-2.0d0 * dlog(zbqlu01(0.0d0)))
      zbqlnor = (r * dcos(theta))
      spare = (r * dsin(theta))
      status = 1
    else
      zbqlnor = spare
      status = 0
    endif

    zbqlnor = mu + (sigma * zbqlnor)

  end
