% This script reads in statistics output by diablo


% Read inputs:
set_inputs;

if recycle
    return
elseif exist([full_dir '/mean.mat'],'file')
    load([full_dir '/mean.mat']);
    return
end

% Set the start and end time in code units for start of averaging
tstart = 0;

% Set the filename
filename = [full_dir '/mean.h5'];

% Read in the number of samples (nk)
file_info = h5info(filename);
att_info = file_info.Groups.Attributes;
Nt = att_info.Value - 2;
fprintf('%i Samples Available (minus 2) \n',Nt)


% Preallocate variables for speed
time = zeros(1,Nt);
ume = zeros(Nz,Nt); vme = zeros(Nz,Nt); wme = zeros(Nz,Nt);
urms = zeros(Nz,Nt); vrms = zeros(Nz,Nt); wrms = zeros(Nz,Nt);
uv = zeros(Nz,Nt); uw = zeros(Nz,Nt); wv = zeros(Nz,Nt);
dudz = zeros(Nz,Nt); dvdz = zeros(Nz,Nt); cp = zeros(Nz,Nt); shear = zeros(Nz,Nt);
omega_x = zeros(Nz,Nt); omega_y = zeros(Nz,Nt); omega_z = zeros(Nz,Nt);
thme = zeros(Nz,Nt,N_th); dthdz = zeros(Nz,Nt,N_th); thrms = zeros(Nz,Nt,N_th);
thw = zeros(Nz,Nt,N_th); pe_diss = zeros(Nz,Nt,N_th);
bpe = zeros(1,Nt);

nu_sgs = zeros(Nz,Nt);
epsilon_sgs = zeros(Nz,Nt);

epsilon = zeros(Nz,Nt);
FTx_uu = zeros(Nx/2,Nt);


clear gzf;
for k = 1:Nt
    timename = sprintf('%04i',k);
    varname = ['/time/' timename];            % TIME
    time(k) = h5read(filename,varname);
    varname = ['/gzf/' timename];             % Y-COORDINATE
    if k == 1,  gzf(:) = h5read(filename,varname); end

% Here, read in the statistics calculated in save_stats
% If you add new output variables to save_stats, simply add
% lines here to read in the variables, following the examples below

    % FROM MEAN.H

    varname = ['/ume/' timename];             % MEAN VELOCITIES
    ume(:,k) = h5read(filename,varname);
    varname = ['/vme/' timename];
    vme(:,k) = h5read(filename,varname);
    varname = ['/wme/' timename];
    wme(:,k) = h5read(filename,varname);
    varname = ['/urms/' timename];            % RMS VELOCITIES
    urms(:,k) = h5read(filename,varname);
    varname = ['/vrms/' timename];
    vrms(:,k) = h5read(filename,varname);
    varname = ['/wrms/' timename];
    wrms(:,k) = h5read(filename,varname);
    varname = ['/uv/' timename];              % REYNOLDS STRESSES
    uv(:,k) = h5read(filename,varname);
    varname = ['/uw/' timename];
    uw(:,k) = h5read(filename,varname);
    varname = ['/wv/' timename];
    wv(:,k) = h5read(filename,varname);
    varname = ['/dudz/' timename];            % MEAN VELOCITY GRADIENTS
    dudz(:,k) = h5read(filename,varname);
    varname = ['/dvdz/' timename];
    dvdz(:,k) = h5read(filename,varname);
    varname = ['/cp/' timename];              % MEAN PRESSURE
    cp(:,k) = h5read(filename,varname);
    varname = ['/shear/' timename];           % MEAN SQUARE SHEAR
    shear(:,k) = h5read(filename,varname);
    varname = ['/omega_x/' timename];         % _RMS_ VORTICITIES
    omega_x(:,k) = h5read(filename,varname);
    varname = ['/omega_y/' timename];
    omega_y(:,k) = h5read(filename,varname);
    varname = ['/omega_z/' timename];
    omega_z(:,k) = h5read(filename,varname);
    varname = ['/epsilon/' timename];             % DISSIPATION RATE
    epsilon(:,k) = h5read(filename,varname);
        epsilon(1,k) = 0; % On GY grid
    varname = ['/FTx_uu/' timename];
    FTx_uu(:,k) = h5read(filename,varname);

    if N_th ~= 1
        error('Warning: Not implemented with N_th > 1')
    end
    n = 1;
        varname = ['/thme' num2str(n,'%2.2d') '/' timename];  % MEAN BUOYANCY
        thme(:,k) = h5read(filename,varname);
        varname = ['/dthdz' num2str(n,'%2.2d') '/' timename]; % MEAN BUOYANCY GRADIENTS
        dthdz(:,k) = h5read(filename,varname);
        varname = ['/thrms' num2str(n,'%2.2d') '/' timename]; % RMS BUOYANCY
        thrms(:,k) = h5read(filename,varname);
        varname = ['/thw' num2str(n,'%2.2d') '/' timename];
        thw(:,k) = h5read(filename,varname);
        varname = ['/pe_diss' num2str(n,'%2.2d') '/' timename]; % POTENTIAL ENERGY DISSIPATION
        pe_diss(:,k) = h5read(filename,varname);


    varname = ['/BPE/' timename];            % MEAN BPE (NOT Vertically-integrated!)
    bpe(k) = h5read(filename,varname);


    % FROM LES OUTPUT (During RK_STEP 1)
    if LES
        varname = ['/nu_sgs/' timename];
        nu_sgs(:,k) = h5read(filename,varname);
        varname = ['/eps_sgs1/' timename];
        eps_sgs1 = -1 * h5read(filename,varname);
        varname = ['/eps_sgs2/' timename];
        eps_sgs2 = -1 * h5read(filename,varname);
            epsilon_sgs(:,k) = eps_sgs1 + eps_sgs2;
            epsilon_sgs(1,k) = 0;  % On GY grid
    else
        nu_sgs(:,k) = zeros(Nz,1);
        epsilon_sgs(:,k) = zeros(Nz,1);
    end



end

% Compute secondary quantities
tii = time; % tii is a shorthand for time

tke = 0.5*(urms.^2 + vrms.^2 + wrms.^2);
nu_t = -uw./dudz;
    nu_t(dudz == 0) = 0;
taylor = sqrt((ume.^2 + vme.^2 + urms.^2 + vrms.^2)./shear); % Vertical Taylor Scale
    taylor(shear == 0) = 0;

brunt = zeros([Nz,Nt,N_th]);
grarich = zeros([Nz,Nt,N_th]);
kappa_t = zeros([Nz,Nt,N_th]);
ape_lorentz = zeros([Nz,Nt,N_th]);

if (N_th > 0)
    if N_th ~= 1
        error('Warning: Not implemented with N_th > 1')
    end
    n = 1;
        brunt(:,:,n) = sqrt((dthdz(:,:,n))); % Brunt-Vaisalla (buoyancy) frequency
        grarich(:,:,n) = brunt(:,:,n).^2./shear; % Gradient richardson number
            grarich(shear == 0) = 0;
        kappa_t(:,:,n) = -thw(:,:,n)./dthdz(:,:,n); % Resolved turbulent diffusivity
            kappa_t(dthdz(:,:,n) == 0) = 0;
        ape_lorentz(:,:,n) = thrms(:,:,n).^2./dthdz(:,:,n); % Perturbation APE (Lorentz 1955) -- Only for small b'

end

% Get the time index based on start time
kstart = 0;
for k = 1:Nt
  if (time(k) <=  tstart)
     kstart = k;
  end
end
if (kstart  ==  0)
  kstart = 1;
end
%'Start of time averaging window: ',time(kstart)

% Get the time index based on end time (if defined)
if exist('tend','var')
    kend = 0;
    for k = 1:Nt
      if (time(k) <=  tend)
         kend = k;
      end
    end
    if (kend  ==  0)
      kend = 1;
    end
else
    kend = Nt;
end
%'End of time averaging window: ',time(kend)

dt_avg = time(kend) - time(kstart);

ume_mean = trapz(time(kstart:kend),ume(:,kstart:kend),2)'/dt_avg;
vme_mean = trapz(time(kstart:kend),vme(:,kstart:kend),2)'/dt_avg;
wme_mean = trapz(time(kstart:kend),wme(:,kstart:kend),2)'/dt_avg;
urms_mean = trapz(time(kstart:kend),urms(:,kstart:kend),2)'/dt_avg;
vrms_mean = trapz(time(kstart:kend),vrms(:,kstart:kend),2)'/dt_avg;
wrms_mean = trapz(time(kstart:kend),wrms(:,kstart:kend),2)'/dt_avg;
dudz_mean = trapz(time(kstart:kend),dudz(:,kstart:kend),2)'/dt_avg;
dvdz_mean = trapz(time(kstart:kend),dvdz(:,kstart:kend),2)'/dt_avg;
tke_mean = trapz(time(kstart:kend),tke(:,kstart:kend),2)'/dt_avg;
uw_mean = trapz(time(kstart:kend),uw(:,kstart:kend),2)'/dt_avg;
wv_mean = trapz(time(kstart:kend),wv(:,kstart:kend),2)'/dt_avg;
cp_mean = trapz(time(kstart:kend),cp(:,kstart:kend),2)'/dt_avg;
omega_x_mean = trapz(time(kstart:kend),omega_x(:,kstart:kend),2)'/dt_avg;
omega_y_mean = trapz(time(kstart:kend),omega_y(:,kstart:kend),2)'/dt_avg;
omega_z_mean = trapz(time(kstart:kend),omega_z(:,kstart:kend),2)'/dt_avg;
shear_mean = trapz(time(kstart:kend),shear(:,kstart:kend),2)'/dt_avg;
epsilon_mean = trapz(time(kstart:kend),epsilon(:,kstart:kend),2)'/dt_avg;

nu_t_mean = -uw_mean./dudz_mean;
    nu_t_mean(dudz_mean == 0) = 0;

thw_mean = zeros([Nz,N_th]);
dthdz_mean = zeros([Nz,N_th]);
thrms_mean = zeros([Nz,N_th]);
thme_mean = zeros([Nz,N_th]);
pe_diss_mean = zeros([Nz,N_th]);
kappa_t_mean = zeros([Nz,N_th]);

if N_th ~= 1
    error('Warning: Not implemented with N_th > 1')
end
n = 1;
    thw_mean(:,n) = trapz(time(kstart:kend),thw(:,kstart:kend,n),2)'/dt_avg;
    dthdz_mean(:,n) = trapz(time(kstart:kend),dthdz(:,kstart:kend,n),2)/dt_avg;
    thrms_mean(:,n) = trapz(time(kstart:kend),thrms(:,kstart:kend,n),2)/dt_avg;
    thme_mean(:,n) = trapz(time(kstart:kend),thme(:,kstart:kend,n),2)/dt_avg;
    pe_diss_mean(:,n) = trapz(time(kstart:kend),pe_diss(:,kstart:kend,n),2)/dt_avg;
    kappa_t_mean(:,n) = -thw_mean(:,n)./dthdz_mean(:,n);
        kappa_t_mean(dthdz_mean(:,n) == 0) = 0;



nu_sgs_mean = trapz(time(kstart:kend),nu_sgs(:,kstart:kend),2)'/dt_avg;
epsilon_sgs_mean = trapz(time(kstart:kend),epsilon_sgs(:,kstart:kend),2)'/dt_avg;


make_grids;


% Calculate z-integrated quantities
urms_int = trapz(gzf,urms,1);
vrms_int = trapz(gzf,vrms,1);
wrms_int = trapz(gzf,wrms,1);
tke_int = trapz(gzf,tke,1);
hke_int = trapz(gzf,(urms.^2+vrms.^2)/2,1);
vke_int = trapz(gzf,wrms.^2/2,1);

ape_lorentz_int = zeros([Nt,N_th]);

for n = 1:N_th
  ape_lorentz_int(:,n) = Ri(n)*trapz(gzf,thrms(:,:,n).^2,1)./(thme(end,:,n)-thme(1,:,n))/2;
end
thw_int = trapz(gzf,thw,1);
thrms_int = trapz(gzf,thrms,1);
epsilon_int = trapz(gzf,epsilon,1);

nu_sgs_int = trapz(gzf,nu_sgs,1);
epsilon_sgs_int = trapz(gzf,epsilon_sgs,1);


clear recycle j k n kstart kend linenum f_input timename varname C att_info ans

%mean_variables = who; sprintf('''%s'',',mean_variables{:})
save([full_dir '/mean.mat'],'FTx_uu','IC_type','LES','Lx','Ly','Lz','N_th','Nt','Nx','Ny','Nz','Pr','Re','Re_v','Ri','Ro','ape_lorentz','ape_lorentz_int','base_dir','beta','bpe','brunt','cp','cp_mean','dTHdx','dTHdy','dVdx','dVdz','delta','dt_avg','dthdz','dthdz_mean','dudz','dudz_mean','dvdz','dvdz_mean','dx','dy','dz','dzf','epsilon','epsilon_int','epsilon_mean','epsilon_sgs','epsilon_sgs_int','epsilon_sgs_mean','file_info','filename','full_dir','grarich','gz','gzf','hke_int','is3D','kappa','kappa_t','kappa_t_mean','nu','nu_sgs','nu_sgs_int','nu_sgs_mean','nu_t','nu_t_mean','nu_v','omega_x','omega_x_mean','omega_y','omega_y_mean','omega_z','omega_z_mean','pe_diss','pe_diss_mean','perturbation','root_dir','rz','shear','shear_mean','taylor','thme','thme_mean','thrms','thrms_int','thrms_mean','thw','thw_int','thw_mean','tii','time','tke','tke_int','tke_mean','tstart','ume','ume_mean','urms','urms_int','urms_mean','uv','uw','uw_mean','vke_int','vme','vme_mean','vrms','vrms_int','vrms_mean','wme','wme_mean','wrms','wrms_int','wrms_mean','wv','wv_mean','x','y');
