% This script shows how to load in 2D slices and make a movie of the simulation output
close all; clear all;

set_inputs;
make_grids;

mkdir(full_dir, '/movie');

% Set the filename
filename = [full_dir '/movie.h5'];
file_info = h5info(filename);
att_info = file_info.Groups.Attributes;
Nt = att_info.Value - 2;
fprintf('%i Samples Available (minus 2) \n',Nt)



rdbu = flipud(cbrewer('div', 'RdBu', 128));
rd = cbrewer('seq', 'Reds', 128);


%cmax = 1e-2%1/(Ro*delta);
tstart = 0  *  (Ro*delta)

parfor k = 1:Nt
    timename = sprintf('%04i',k);

    time = h5readatt(filename,['/u_xy/' timename],'Time');

    if exist([full_dir '/movie/XYcomposite' sprintf('%04i', k) '.png'],'file')
        continue;
    end

    f = figure();
    ax = subplots(2,2,[0.015 0.035],[0.085,0.1],[0.06,0.05]);
    figtitle = sgtitle(''); figtitle.FontSize = 12;
    figtitle.String = sprintf('$t = %.2f \\; [1/f]$',time/(Ro*delta));

    if time < tstart - 1e-4
        close(f);
        continue;
    end

    A_b = h5read(filename,['/th1_xy/' timename]);
    A_u = h5read(filename,['/u_xy/' timename]);
    A_v = h5read(filename,['/v_xy/' timename]);
    A_w = h5read(filename,['/w_xy/' timename]);


    A_zvort = ddx(A_v,dx) - ddy(A_u,dy);

    %A_Ri_g = ddz(A_b,dzf,1) ./  ( ddz(A_u,dzf,1).^2 + ddz(A_w,dzf,0).^2);



    %dat = A_w.*A_b; % Buoyancy Flux
    %dat = A_Ri_g;
    dat = A_u;
    axes(ax(1));
    pcolor(x,y,dat'); shading interp;
    colourbar('$u$',rdbu,'Centred');
    ylabel('$y$')

    %dat = A_w.*A_v; % GS Momentum Flux (v'w')
    dat = A_w;
    axes(ax(2));
    pcolor(x,y,dat'); shading interp;
    colourbar('$w$',rdbu,'Centred');

    dat = A_b; % Buoyancy
    axes(ax(3));
    pcolor(x,y,dat'); shading interp;
    colourbar('$b''$',rdbu,'Centred');
    xlabel('$x$')
    ylabel('$y$')

    dat = A_zvort; % Vorticity
    axes(ax(4));
    pcolor(x,y,dat'); shading interp;
    colourbar('$\omega_z$',rdbu,[-2*rms(dat(:)),2*rms(dat(:))]);
    xlabel('$x$')



    set(ax(1:2),'XTickLabel',[]);
    set(ax([2,4]),'YTickLabel',[]);
    set(ax(1:4),'layer','top');


    save_fig(f,[full_dir '/movie/XYcomposite' sprintf('%04i', k) '.png'])

    close(f);


end


%!ffmpeg -r 30 -f image2 -s 1280x720 -i ./movie/composite%04d.png -vcodec libx264 -crf 25  -pix_fmt yuv420p -vf "pad=ceil(iw/2)*2:ceil(ih/2)*2:color=white" composite.mp4
